import forbiddenfruit as ff 

def parseInt(self) -> int:
    try:
        return int(self)
    except: return None

def parseFloat(self) -> float:
    try:
        return float(self)
    except: return None

class Number:
    def __init__(self, nmr=None):
        try:
            self.num = float(nmr)
        except:
            self.num = None
        
    def __float__(self):
        return self.num
    
    def __int__(self):
        return self.num
    
    def __str__(self):
        return str(self.num)

    def MAX_VALUE() -> complex:
        return 1.7976931348623157e+308

    def MIN_VALUE() -> complex:
        return 5e-324

    def MAX_SAFE_INTEGER() -> int:
        return 9007199254740991
    
    def MIN_SAFE_INTEGER() -> int:
        return -9007199254740991


import forbiddenfruit as ff

def num_toString(self) -> str:
    return str(self)

def num_toExponential(self, N=1) -> str:
    return format(self, f'0.{N}e')

def num_toFixed(self, N=1) -> str:
    return format(self, f"0.{N}f")

def num_valueOf(self):
    return self

ff.curse(float, "toString", num_toString)
ff.curse(int, "toString", num_toString)
ff.curse(int, "toExponential", num_toExponential)
ff.curse(float, "toExponential", num_toExponential)
ff.curse(int, "toFixed", num_toFixed)
ff.curse(float, "toFixed", num_toFixed)
ff.curse(int, "valueOf", num_valueOf)
ff.curse(float, "valueOf", num_valueOf)


import forbiddenfruit as ff
from typing import Union

def ar_toString(self:Union[list, tuple]) -> str:
    return ','.join(str(i) for i in self)

def ar_join(self:Union[list, tuple], connector:str=",") -> str:
    return connector.join(str(i) for i in self)

ff.curse(list, "toString", ar_toString)
ff.curse(tuple, "toString", ar_toString)
ff.curse(list, "join", ar_join)
ff.curse(tuple, "join", ar_join)


import forbiddenfruit as ff

@property
def str_length(self) -> int:
    return len(self)

def str_search(self, other) -> int:
    return self.find(other)

def str_indexOf(self, other, startPos:int=0) -> int:
    return self[startPos:].find(other)

def str_lastIndexOf(self, other, startPos:int=0) -> int:
    return self[startPos:].rfind(other)

def str_slice(self, start=None, end=None) -> str:
    if not start:
        return self
    if not end:
        return self[start]
    return self[start:end]

def str_substring(self, start=None, end=None) -> str:
    if not start:
        return self
    if not end:
        if start < 0: raise ValueError("str.substring() cannot take negative indexes.")
        return self[start]
    if start < 0 or end < 0: raise ValueError("str.substring() cannot take negative indexes.")
    return self[start:end]

def str_toUpperCase(self) -> str:
    return self.upper()

def str_toLowerCase(self) -> str:
    return self.lower()

def str_concat(self, *other) -> str:
    return self + ''.join(other)

def str_trim(self) -> str:
    return self.strip()

def str_padStart(self, maxLength:int, fillString) -> str:
    return str(fillString)*(maxLength-1) + self

def str_padEnd(self, maxLength:int, fillString) -> str:
    return self + str(fillString)*(maxLength-1)

def str_charCodeAt(self, index=0) -> int:
    return ord(self[index])

ff.curse(str, "length", str_length)
ff.curse(str, "indexOf", str_indexOf)
ff.curse(str, "lastIndexOf", str_lastIndexOf)
ff.curse(str, "search", str_search)
ff.curse(str, "slice", str_slice)
ff.curse(str, "substring", str_substring)
ff.curse(str, "toUpperCase", str_toUpperCase)
ff.curse(str, "toLowerCase", str_toLowerCase)
ff.curse(str, "concat", str_concat)
ff.curse(str, "trim", str_trim)
ff.curse(str, "padStart", str_padStart)
ff.curse(str, "padEnd", str_padEnd)
ff.curse(str, "charCodeAt", str_charCodeAt)


class createReadStream:
    def __init__(self, path:str):
        self.path = path

    def chunk(self):
        return open(self.path, "r").read()
    
class createWriteStream:
    def __init__(self, path:str):
        self.path = path

    def write(self, _data:str):
        with open(self.path, "w+") as f:
            f.write(_data)